"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var types_1 = require("./types");
var react_transition_group_1 = require("react-transition-group");
/**
 * @prop show: boolean
 * @prop direction: like ISliderDirection 0: left, 1: right, 2: up, 3: down
 * @prop withFade?: boolean
 * @prop fixedDistance?: revealTill a distance and show scrollbar
 * @prop childStyles?: all lise ISliderChildStyles except width or height
 * @prop mountUnmountAuthority
 */
var Reveal = /** @class */ (function (_super) {
    __extends(Reveal, _super);
    function Reveal() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.checkBeforeRender = function () {
            var _a = _this.props.childStyles, transitionTime = _a.transitionTime, enterTransitionTime = _a.enterTransitionTime, exitTransitionTime = _a.exitTransitionTime;
            if (!transitionTime) {
                if (!(enterTransitionTime && exitTransitionTime)) {
                    throw new Error("If transition time is not defined then both enter transition time and exit transition time should be defined");
                }
            }
        };
        _this.cssHeightWidthPropWithValue = function (forMax) {
            var styles = {};
            if (_this.props.direction === types_1.IRevealDirection.MoveDown) {
                if (forMax) {
                    styles = { maxHeight: "100vh" };
                }
                else {
                    styles = {
                        maxHeight: _this.props.childStyles.minMeasureValue || "0px"
                    };
                }
            }
            else if (_this.props.direction === types_1.IRevealDirection.MoveRight) {
                if (forMax) {
                    styles = { maxWidth: "100vw" };
                }
                else {
                    styles = {
                        maxWidth: _this.props.childStyles.minMeasureValue || "0px"
                    };
                }
            }
            else {
                throw new Error("Either direction prop is not present or it's value provided is wrong");
            }
            return styles;
        };
        return _this;
    }
    Reveal.prototype.render = function () {
        var _this = this;
        this.checkBeforeRender();
        var _a = this.props, show = _a.show, mountUnmountAuthority = _a.mountUnmountAuthority, children = _a.children, withFade = _a.withFade, revealOnAppear = _a.revealOnAppear;
        return (React.createElement(react_transition_group_1.Transition, { in: show, mountOnEnter: mountUnmountAuthority, unmountOnExit: mountUnmountAuthority, timeout: {
                enter: 1,
                exit: this.exitTransitionTime * 1000
            }, appear: revealOnAppear || false }, function (state) {
            var wrapperStyle;
            switch (state) {
                case "entering":
                    wrapperStyle = __assign({}, _this.cssHeightWidthPropWithValue(false), { opacity: withFade ? 0 : 1 });
                    break;
                case "entered":
                    wrapperStyle = __assign({}, _this.cssHeightWidthPropWithValue(true), { opacity: 1, transition: _this.cssDirection + " " + _this.enterTransitionTime + "s " + _this.enterTimingFunction + ", opacity " + _this.enterTransitionTime + "s " + _this.enterTimingFunction });
                    break;
                case "exiting":
                    wrapperStyle = __assign({}, _this.cssHeightWidthPropWithValue(false), { opacity: withFade ? 0 : 1, transition: _this.cssDirection + " " + _this.exitTransitionTime + "s " + _this.exitTimingFunction + ", opacity " + _this.exitTransitionTime + "s " + _this.exitTimingFunction });
                    break;
                case "exited":
                    wrapperStyle = __assign({}, _this.cssHeightWidthPropWithValue(false), { opacity: withFade ? 0 : 1 });
                    break;
            }
            var allStyles = __assign({}, wrapperStyle, _this.normalStyles);
            return React.createElement("div", { style: allStyles }, children);
        }));
    };
    Object.defineProperty(Reveal.prototype, "enterTransitionTime", {
        get: function () {
            var _a = this.props.childStyles, transitionTime = _a.transitionTime, enterTransitionTime = _a.enterTransitionTime;
            return enterTransitionTime || transitionTime || 1;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Reveal.prototype, "exitTransitionTime", {
        get: function () {
            var _a = this.props.childStyles, transitionTime = _a.transitionTime, exitTransitionTime = _a.exitTransitionTime;
            return exitTransitionTime || transitionTime || 1;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Reveal.prototype, "enterTimingFunction", {
        get: function () {
            var _a = this.props.childStyles, timingFunction = _a.timingFunction, enterTimingFunction = _a.enterTimingFunction;
            return enterTimingFunction || timingFunction || "linear";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Reveal.prototype, "exitTimingFunction", {
        get: function () {
            var _a = this.props.childStyles, timingFunction = _a.timingFunction, exitTimingFunction = _a.exitTimingFunction;
            return exitTimingFunction || timingFunction || "linear";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Reveal.prototype, "normalStyles", {
        get: function () {
            return {
                overflow: "hidden"
            };
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Reveal.prototype, "cssDirection", {
        get: function () {
            if (this.props.direction === types_1.IRevealDirection.MoveDown) {
                return "max-height";
            }
            else if (this.props.direction === types_1.IRevealDirection.MoveRight) {
                return "max-width";
            }
            else {
                throw new Error("Either direction prop is not present or it's value provided is wrong");
            }
        },
        enumerable: true,
        configurable: true
    });
    return Reveal;
}(React.Component));
exports.default = Reveal;
//# sourceMappingURL=Reveal.js.map