"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var styles_1 = require("./styles");
var TransitioningComponent_1 = require("./TransitioningComponent");
var util_1 = require("util");
var utils_1 = require("./utils");
/**
 * @prop watchProp - The property to watch if on which sliding will occur. This must be a literal value.
 * @prop childProps - The properties of child element.
 * @prop direction - The direction in which this change's sliding effect should go.
 * @prop children - This child JSX element wrapped in Slider component on which sliding effect will happen.
 * @prop childStyles - Some styles of children, which includes width, height and transition timing related styles.
 * @prop slideOnAppear - If we want the sliding effect when we first see the screen. Optional Prop.
 * @prop fadeOnSlide - Should the slide fade on entering or exiting. Optional Prop.
 * @prop sizePercentageDuringSlide - % of size which should be on screen. Useful only with fadeOnSlide prop. Optional.
 * @prop transitionDone - Transition done callback. Optional.
 */
var Slider = /** @class */ (function (_super) {
    __extends(Slider, _super);
    function Slider(props) {
        var _this = _super.call(this, props) || this;
        _this.transitionCycle = "full" /* Full */;
        _this.selfRef = null;
        _this.firstRender = true;
        _this.curWatchCount = 0;
        _this.prevWatchCount = 0;
        _this.enterTransitionDone = function () {
            _this.transitionCycle = "full" /* Full */;
            if (_this.props.transitionDone) {
                _this.props.transitionDone();
            }
        };
        /**
         * This method makes two clones of child.
         * One with current properties which will enter and another with previous properties which will exit.
         */
        _this.getCLonedElems = function () {
            var _a = _this.state, prevWatchProp = _a.prevWatchProp, prevChildProps = _a.prevChildProps, nextWatchProp = _a.nextWatchProp, nextChildProps = _a.nextChildProps;
            var _b = _this.props, direction = _b.direction, childStyles = _b.childStyles, slideOnAppear = _b.slideOnAppear, fadeOnSlide = _b.fadeOnSlide, sizePercentageDuringSlide = _b.sizePercentageDuringSlide, children = _b.children;
            var clonedElems = [];
            var enterTimeout = (childStyles.transitionTime || childStyles.enterTransitionTime || 1) *
                100;
            var exitTimeout = (childStyles.transitionTime || childStyles.exitTransitionTime || 1) * 100;
            var preparedChildStylesEnter = __assign({}, childStyles, { zIndex: 2 });
            if (utils_1.noNullOrUndefined(nextWatchProp) && nextChildProps) {
                clonedElems.push(React.createElement(TransitioningComponent_1.default, { enter: true, direction: direction, key: "s_" + _this.curWatchCount + nextWatchProp, appear: slideOnAppear || utils_1.noNullOrUndefined(prevWatchProp), parentRef: _this.selfRef, childStyles: preparedChildStylesEnter, fadeOnSlide: fadeOnSlide, sizePercentageDuringSlide: sizePercentageDuringSlide, timeout: enterTimeout, transitionEndCallback: _this.enterTransitionDone }, React.cloneElement(children, nextChildProps)));
            }
            var preparedChildStylesExit = __assign({}, childStyles, { zIndex: 1 });
            if (utils_1.noNullOrUndefined(prevWatchProp) &&
                prevWatchProp !== nextWatchProp &&
                prevChildProps) {
                clonedElems.push(React.createElement(TransitioningComponent_1.default, { enter: false, direction: direction, key: "s_" + _this.prevWatchCount + prevWatchProp, appear: true, parentRef: _this.selfRef, childStyles: preparedChildStylesExit, fadeOnSlide: fadeOnSlide, sizePercentageDuringSlide: sizePercentageDuringSlide, timeout: exitTimeout }, React.cloneElement(children, prevChildProps)));
            }
            return clonedElems;
        };
        _this.state = {
            nextChildProps: null,
            nextWatchProp: null,
            prevChildProps: null,
            prevWatchProp: null
        };
        return _this;
    }
    Slider.getDerivedStateFromProps = function (nextProps, prevState) {
        return {
            nextChildProps: nextProps.childProps,
            nextWatchProp: nextProps.watchProp,
            prevChildProps: prevState.nextChildProps,
            prevWatchProp: prevState.nextWatchProp
        };
    };
    Slider.prototype.shouldComponentUpdate = function (nextProps) {
        var validTransitionCycleState = this.transitionCycle === "full" /* Full */ ||
            this.transitionCycle === "start" /* Start */;
        var isWatchPropDifferent = this.props.watchProp !== nextProps.watchProp;
        return validTransitionCycleState && isWatchPropDifferent;
    };
    Slider.prototype.render = function () {
        var _this = this;
        this.beforeUpdationProcess();
        var _a = this.props.childStyles, width = _a.width, height = _a.height;
        var rtgListStyles = styles_1.default().rtgList;
        var styles = __assign({}, rtgListStyles, { width: width, height: height });
        return (React.createElement("div", { ref: function (elem) { return (_this.selfRef = elem); }, style: styles, className: this.props.className }, this.getCLonedElems()));
    };
    Slider.prototype.beforeUpdationProcess = function () {
        var _a = this.props, children = _a.children, watchProp = _a.watchProp;
        if (!React.isValidElement(children)) {
            throw new Error("Wrapped child is not a valid react element");
        }
        if (Array.isArray(children)) {
            throw new Error("Only single child can be passed in slider component");
        }
        if (typeof children === "string") {
            throw new Error("Wrapped child cannot be string, it should be a single react element");
        }
        if (typeof watchProp === "undefined" || util_1.isNull(watchProp)) {
            throw new Error("**watchProp** cannot be null or undefined");
        }
        if (watchProp === Object(watchProp)) {
            throw new Error("**watchProp** must be either string, number or boolean");
        }
        if (this.transitionCycle !== "full" /* Full */) {
            console.warn("Slider(React-Animation-Suite): You should not change properties unless transition cycle is fully complete.\n        Please manage this in your code by using **transitionDone** callback prop");
        }
        if (!this.firstRender) {
            this.transitionCycle = "start" /* Start */;
        }
        else {
            this.firstRender = false;
        }
        if (this.state.nextWatchProp !== this.state.prevWatchProp) {
            this.prevWatchCount = this.curWatchCount;
            this.curWatchCount += 1;
        }
    };
    return Slider;
}(React.Component));
exports.default = Slider;
//# sourceMappingURL=Slider.js.map