"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
        return extendStatics(d, b);
    }
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var styles_1 = require("./styles");
var TransitioningComponentChild_1 = require("./TransitioningComponentChild");
var react_transition_group_1 = require("react-transition-group");
var TransitioningComponent = /** @class */ (function (_super) {
    __extends(TransitioningComponent, _super);
    function TransitioningComponent() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        // Transition complete function
        // tslint:disable-next-line:no-shadowed-variable
        _this.addEndListener = function (node, done) {
            node.addEventListener("transitionend", function () {
                done();
                if (_this.props.transitionEndCallback) {
                    _this.props.transitionEndCallback();
                }
            }, false);
        };
        return _this;
    }
    TransitioningComponent.prototype.render = function () {
        var _this = this;
        var _a = this.props, enter = _a.enter, appear = _a.appear, children = _a.children, fadeOnSlide = _a.fadeOnSlide, timeout = _a.timeout;
        // These constants are being put here because the getter properties could get exact value before transition starts.
        // Putting it inside transition rendered block could produce undeterinistic values.
        var partialTransitioningComponentChildProps = {
            child: children,
            cssWrapperStyle: this.cssWrapperStyle,
            enteringTransform: this.enteringTransform,
            exitingTransform: this.exitingTransform,
            fadeOnSlide: fadeOnSlide || false,
            sliderEnterTimingFunction: this.sliderEnterTimingFunction,
            sliderExitTimingFunction: this.sliderExitTimingFunction,
            sliderTransitionEnterTime: this.sliderTransitionEnterTime,
            sliderTransitionExitTime: this.sliderTransitionExitTime,
            wrapperTransformCenter: this.wrapperTransformCenter
        };
        var renderState = function (state) {
            return _this.renderTransitioningChild(__assign({}, partialTransitioningComponentChildProps, { state: state }));
        };
        return (React.createElement(react_transition_group_1.Transition, { in: enter, timeout: timeout, appear: appear, addEndListener: this.addEndListener }, renderState));
    };
    Object.defineProperty(TransitioningComponent.prototype, "sliderHorizontalMargin", {
        // Getter method for calculating horizontal margin
        // Useful when sliding direction is right/left.
        get: function () {
            var _a = this.props, parentRef = _a.parentRef, childStyles = _a.childStyles;
            var sliderElemMargin = 0;
            if (parentRef && parentRef.parentElement) {
                sliderElemMargin =
                    parentRef.parentElement.offsetWidth - childStyles.width;
            }
            return sliderElemMargin;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TransitioningComponent.prototype, "sliderVerticalMargin", {
        // Getter method for calculating vertical margin
        // Useful when sliding direction is up/down.
        get: function () {
            var _a = this.props, parentRef = _a.parentRef, childStyles = _a.childStyles;
            var sliderElemMargin = 0;
            if (parentRef && parentRef.parentElement) {
                sliderElemMargin =
                    parentRef.parentElement.offsetHeight - childStyles.height;
            }
            return sliderElemMargin;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TransitioningComponent.prototype, "horizontalFarDistance", {
        // Getter method for calculating the horizontal distance an element must travel.
        // Useful when sliding direction is right/left.
        get: function () {
            var _a = this.props, sizePercentageDuringSlide = _a.sizePercentageDuringSlide, childStyles = _a.childStyles, fadeOnSlide = _a.fadeOnSlide;
            var farDistance = childStyles.width + this.sliderHorizontalMargin;
            if (fadeOnSlide && sizePercentageDuringSlide) {
                farDistance = childStyles.width * (1 - sizePercentageDuringSlide / 100);
            }
            return farDistance;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TransitioningComponent.prototype, "verticalFarDistance", {
        // Getter method for calculating vertical distance an element must travel.
        // Useful when sliding direction is up/down.
        get: function () {
            var _a = this.props, sizePercentageDuringSlide = _a.sizePercentageDuringSlide, childStyles = _a.childStyles, fadeOnSlide = _a.fadeOnSlide;
            var farDistance = childStyles.height + this.sliderVerticalMargin;
            if (fadeOnSlide && sizePercentageDuringSlide) {
                farDistance = childStyles.height * (1 - sizePercentageDuringSlide / 100);
            }
            return farDistance;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TransitioningComponent.prototype, "wrapperStyleFarRight", {
        // Getter method for transform style if component is at right and is invisible.
        // Useful when sliding direction is right/left.
        get: function () {
            return "translate3d(" + this.horizontalFarDistance + "px, 0, 0)";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TransitioningComponent.prototype, "wrapperStyleFarLeft", {
        // Getter method for transform style if component is at left and is invisible.
        // Useful when sliding direction is right/left.
        get: function () {
            return "translate3d(-" + this.horizontalFarDistance + "px, 0, 0)";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TransitioningComponent.prototype, "wrapperStyleFarUp", {
        // Getter method for transform style if component is at up and is invisible.
        // Useful when sliding direction is up/down.
        get: function () {
            return "translate3d(0, -" + this.verticalFarDistance + "px, 0)";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TransitioningComponent.prototype, "wrapperStyleFarDown", {
        // Getter method for transform style if component is at down and is invisible.
        // Useful when sliding direction is up/down.
        get: function () {
            return "translate3d(0, " + this.verticalFarDistance + "px, 0)";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TransitioningComponent.prototype, "wrapperTransformCenter", {
        // Getter method for transform style if component is at center.
        get: function () {
            return "translate3d(0, 0, 0)";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TransitioningComponent.prototype, "sliderTransitionExitTime", {
        // Getter method to calculate transition exit time.
        get: function () {
            var _a = this.props.childStyles, transitionTime = _a.transitionTime, exitTransitionTime = _a.exitTransitionTime;
            var time = exitTransitionTime || transitionTime;
            if (!time) {
                throw new Error("Either exitTransitionTime or transitionTime should be present as props");
            }
            return time + "s";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TransitioningComponent.prototype, "sliderTransitionEnterTime", {
        // Getter method to calculate transition enter time.
        get: function () {
            var _a = this.props.childStyles, transitionTime = _a.transitionTime, enterTransitionTime = _a.enterTransitionTime;
            var time = enterTransitionTime || transitionTime;
            if (!time) {
                throw new Error("Either enterTransitionTime or transitionTime should be present as props");
            }
            return time + "s";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TransitioningComponent.prototype, "sliderEnterTimingFunction", {
        // Getter method to calculate transition enter timing function css property.
        get: function () {
            var _a = this.props.childStyles, timingFunction = _a.timingFunction, enterTimingFunction = _a.enterTimingFunction;
            return enterTimingFunction || timingFunction || "linear";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TransitioningComponent.prototype, "sliderExitTimingFunction", {
        // Getter method to calculate transition exit timing function css property.
        get: function () {
            var _a = this.props.childStyles, timingFunction = _a.timingFunction, exitTimingFunction = _a.exitTimingFunction;
            return exitTimingFunction || timingFunction || "linear";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TransitioningComponent.prototype, "enteringTransform", {
        // Getter method to calculate css transform of entering element.
        // Calculates according to direction in which it will be transitioning.
        get: function () {
            var direction = this.props.direction;
            switch (direction) {
                case 1 /* MoveRight */:
                    return this.wrapperStyleFarLeft;
                case 0 /* MoveLeft */:
                    return this.wrapperStyleFarRight;
                case 2 /* MoveUp */:
                    return this.wrapperStyleFarDown;
                case 3 /* MoveDown */:
                    return this.wrapperStyleFarUp;
                default:
                    throw new Error("No direction present");
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TransitioningComponent.prototype, "exitingTransform", {
        // Getter method to calculate css transform of exiting element.
        // Calculates according to direction in which it will be transitioning.
        get: function () {
            var direction = this.props.direction;
            switch (direction) {
                case 1 /* MoveRight */:
                    return this.wrapperStyleFarRight;
                case 0 /* MoveLeft */:
                    return this.wrapperStyleFarLeft;
                case 2 /* MoveUp */:
                    return this.wrapperStyleFarUp;
                case 3 /* MoveDown */:
                    return this.wrapperStyleFarDown;
                default:
                    throw new Error("No direction present");
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TransitioningComponent.prototype, "cssWrapperStyle", {
        // Get the wrapper css style, non dependent on state of transition.
        get: function () {
            var rtgWrapperStyles = styles_1.default().rtgWrapper;
            var styles = __assign({}, rtgWrapperStyles, { height: this.props.childStyles.height, width: this.props.childStyles.width });
            var zIndex = this.props.childStyles.zIndex;
            return zIndex ? __assign({}, styles, { zIndex: zIndex }) : styles;
        },
        enumerable: true,
        configurable: true
    });
    // Abstraction of rendering child element. Used inside JSX rendering of child elem.
    TransitioningComponent.prototype.renderTransitioningChild = function (transitioningComponentChildProps) {
        return (React.createElement(TransitioningComponentChild_1.TransitioningComponentChild, __assign({}, transitioningComponentChildProps, { enter: this.props.enter })));
    };
    return TransitioningComponent;
}(React.Component));
exports.default = TransitioningComponent;
//# sourceMappingURL=TransitioningComponent.js.map