import * as React from "react";
import { ISliderChildStyles, ISliderDirection, SliderCycleState } from "./types";
export interface ISliderProps {
    watchProp: string | number | boolean;
    childProps: any;
    direction: ISliderDirection;
    children: JSX.Element;
    childStyles: ISliderChildStyles;
    slideOnAppear?: boolean;
    fadeOnSlide?: boolean;
    sizePercentageDuringSlide?: number;
    className?: string;
    transitionDone?: () => void;
}
interface ISliderState {
    prevWatchProp: any;
    prevChildProps: any;
    nextWatchProp: any;
    nextChildProps: any;
}
/**
 * @prop watchProp - The property to watch if on which sliding will occur. This must be a literal value.
 * @prop childProps - The properties of child element.
 * @prop direction - The direction in which this change's sliding effect should go.
 * @prop children - This child JSX element wrapped in Slider component on which sliding effect will happen.
 * @prop childStyles - Some styles of children, which includes width, height and transition timing related styles.
 * @prop slideOnAppear - If we want the sliding effect when we first see the screen. Optional Prop.
 * @prop fadeOnSlide - Should the slide fade on entering or exiting. Optional Prop.
 * @prop sizePercentageDuringSlide - % of size which should be on screen. Useful only with fadeOnSlide prop. Optional.
 * @prop transitionDone - Transition done callback. Optional.
 */
export default class Slider extends React.Component<ISliderProps, ISliderState> {
    static getDerivedStateFromProps(nextProps: ISliderProps, prevState: ISliderState): {
        nextChildProps: any;
        nextWatchProp: string | number | boolean;
        prevChildProps: any;
        prevWatchProp: any;
    };
    transitionCycle: SliderCycleState;
    private selfRef;
    private firstRender;
    private curWatchCount;
    private prevWatchCount;
    constructor(props: ISliderProps);
    shouldComponentUpdate(nextProps: ISliderProps): boolean;
    render(): JSX.Element;
    private beforeUpdationProcess;
    private enterTransitionDone;
    /**
     * This method makes two clones of child.
     * One with current properties which will enter and another with previous properties which will exit.
     */
    private getCLonedElems;
}
export {};
